"use strict";
/*
 * Copyright 2020 NEM
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.VotingService = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const symbol_sdk_1 = require("symbol-sdk");
const logger_1 = require("../logger");
const LoggerFactory_1 = require("../logger/LoggerFactory");
const BootstrapUtils_1 = require("./BootstrapUtils");
const VotingUtils_1 = require("./VotingUtils");
const logger = LoggerFactory_1.default.getLogger(logger_1.LogType.System);
class VotingService {
    constructor(params) {
        this.params = params;
    }
    async run(presetData, nodeAccount, nodePreset) {
        var _a;
        const symbolServerToolsImage = presetData.symbolServerToolsImage;
        function logVotingKeyFiles(currentVotingFiles) {
            const log = currentVotingFiles
                .map((value) => {
                return `${value.filename}, Voting Public Key ${value.publicKey}, Start Epoch: ${value.startEpoch}, End Epoch: ${value.endEpoch}`;
            })
                .join('\n');
            logger.info(`Current Voting files:\n ${log}`);
        }
        if (nodePreset === null || nodePreset === void 0 ? void 0 : nodePreset.voting) {
            const target = this.params.target;
            const votingKeysFolder = path_1.join(BootstrapUtils_1.BootstrapUtils.getTargetNodesFolder(target, true, nodeAccount.name), presetData.votingKeysDirectory);
            await BootstrapUtils_1.BootstrapUtils.mkdir(votingKeysFolder);
            const votingUtils = new VotingUtils_1.VotingUtils();
            const currentVotingFiles = votingUtils.loadVotingFiles(votingKeysFolder);
            const maxVotingKeyEndEpoch = ((_a = currentVotingFiles[currentVotingFiles.length - 1]) === null || _a === void 0 ? void 0 : _a.endEpoch) || presetData.lastKnownNetworkEpoch - 1;
            if (maxVotingKeyEndEpoch > presetData.lastKnownNetworkEpoch + presetData.votingKeyDesiredEpochLength / 2) {
                logVotingKeyFiles(currentVotingFiles);
                nodeAccount.voting = currentVotingFiles;
                return;
            }
            const votingKeyStartEpoch = maxVotingKeyEndEpoch + 1;
            const votingKeyEndEpoch = maxVotingKeyEndEpoch + presetData.votingKeyDesiredEpochLength;
            const votingAccount = symbol_sdk_1.Account.generateNewAccount(presetData.networkType);
            const votingPrivateKey = votingAccount.privateKey;
            const epochs = votingKeyEndEpoch - votingKeyStartEpoch + 1;
            logger.info(`Creating Voting key file of ${epochs} epochs for node ${nodeAccount.name}. This could take a while!`);
            const privateKeyTreeFileName = `private_key_tree${currentVotingFiles.length + 1}.dat`;
            if (presetData.useExperimentalNativeVotingKeyGeneration) {
                const votingFile = await votingUtils.createVotingFile(votingPrivateKey, votingKeyStartEpoch, votingKeyEndEpoch);
                fs_1.writeFileSync(path_1.join(votingKeysFolder, privateKeyTreeFileName), votingFile);
            }
            else {
                const binds = [`${votingKeysFolder}:/votingKeys:rw`];
                const cmd = [
                    `${presetData.catapultAppFolder}/bin/catapult.tools.votingkey`,
                    `--secret=${votingPrivateKey}`,
                    `--startEpoch=${votingKeyStartEpoch}`,
                    `--endEpoch=${votingKeyEndEpoch}`,
                    `--output=/votingKeys/${privateKeyTreeFileName}`,
                ];
                const userId = await BootstrapUtils_1.BootstrapUtils.resolveDockerUserFromParam(this.params.user);
                const { stdout, stderr } = await BootstrapUtils_1.BootstrapUtils.runImageUsingExec({
                    catapultAppFolder: presetData.catapultAppFolder,
                    image: symbolServerToolsImage,
                    userId: userId,
                    cmds: cmd,
                    binds: binds,
                });
                if (stdout.indexOf('<error> ') > -1) {
                    logger.info(stdout);
                    logger.error(stderr);
                    throw new Error('Voting key failed. Check the logs!');
                }
            }
            logger.warn(`A new Voting File for the node ${nodeAccount.name} has been regenerated! `);
            logger.warn(`Remember to send a Voting Key Link transaction from main ${nodeAccount.main.address} using the Voting Public Key: ${votingAccount.publicKey} with startEpoch: ${votingKeyStartEpoch} and endEpoch: ${votingKeyEndEpoch}`);
            logger.warn('For linking, you can use symbol-bootstrap link command, the symbol cli, or the symbol desktop wallet. ');
            const newVotingKeyFiles = votingUtils.loadVotingFiles(votingKeysFolder);
            logVotingKeyFiles(newVotingKeyFiles);
            nodeAccount.voting = newVotingKeyFiles;
        }
        else {
            logger.info(`Non-voting node ${nodeAccount.name}.`);
        }
    }
}
exports.VotingService = VotingService;
